@@ -3,18 +3,19 @@ import random
 import time
 import copy
 import logging
-import numpy as np
 from shutil import copyfile
 
+import numpy as np
 from ase.db import connect
 from ase import Atoms
 from ase.units import invcm, Hartree, kcal, mol
 from ase.thermochemistry import IdealGasThermo
+import rmsd
+
 from kinbot import geometry
 from kinbot import zmatrix
 from kinbot.stationary_pt import StationaryPoint
 from kinbot import constants
-import rmsd
 
 logger = logging.getLogger('KinBot')
 
@@ -40,6 +41,7 @@ class Conformers:
         self.sconf = -1
 
         self.grid = par['conf_grid']
+        self.diffthrs = par['difference_threshold']
 
         # final geometries of the cyclic search
         self.cyc_conf_geoms = []
@@ -87,6 +89,10 @@ class Conformers:
         # db to be used for skipping conf generation
         self.db = connect('kinbot.db')
 
+        self.imagfreq_threshold = par['imagfreq_threshold']
+        self.flat_ring_dih_angle = par['flat_ring_dih_angle']
+        self.print_warning = True
+
     def generate_ring_conformers(self, cart):
         """
         Generate the conformers of a cyclic structure
@@ -109,7 +115,7 @@ class Conformers:
                 for dih in dihs:
                     val = geometry.calc_dihedral(cart[dih[0]], cart[dih[1]],
                                                  cart[dih[2]], cart[dih[3]])[0]
-                    if abs(val) < 5.:
+                    if abs(val) < self.flat_ring_dih_angle:
                         flat_ring_dih.append(True)
                     else:
                         flat_ring_dih.append(False)
@@ -160,6 +166,9 @@ class Conformers:
     def start_ring_conformer_search(self, index, cart):
         """
         index: number of the conformer
+        In each iteration a given dihedral is changed, and then in the 
+        next one it's fixed and another one is changed, and then two are
+        fixed and the next one is changed, until all are at their desired values
         """
         if self.cyc_conf_index[index] == len(self.cyc_dih_atoms[index]) - 1:
             # this conformer has finished
@@ -201,13 +210,14 @@ class Conformers:
                 return geom, -1
             else:
                 # check if all the bond lenghts are withing 10% or the original bond lengths
-                temp = StationaryPoint('temp',
+                dummy = StationaryPoint('dummy',
                                        self.species.charge,
                                        self.species.mult,
                                        atom=self.species.atom,
                                        geom=geom)
-                temp.bond_mx()
-                if geometry.equal_geom(self.species, temp, 0.10):
+                dummy.bond_mx()
+                dummy.calc_chemid()
+                if geometry.equal_geom(self.species, dummy, 0.10):
                     logger.debug('Successfully finished conformer {}'.format(job))
                     return geom, 0
                 else:
@@ -328,7 +338,6 @@ class Conformers:
 
         return 0
 
-
     def generate_conformers_random_sampling(self, ini_cart):
         """
         Generate a random sampling of each dihedral for a number nconfs of conformers
@@ -380,6 +389,7 @@ class Conformers:
                                     atom=self.species.atom,
                                     geom=geom)
             dummy.bond_mx()
+            dummy.calc_chemid()
             if geometry.equal_geom(self.species, dummy, 0.10):
                 return geom, 0
             else:
@@ -410,12 +420,42 @@ class Conformers:
                     status[i] = self.test_conformer(i)[1]
             # problem: if the first sample has 2 imaginary frequencies, what to do then?
             if all([si >= 0 for si in status]):
-                lowest_totenergy = 0.
-                lowest_e_geom = self.species.geom
+                if isinstance(name, int):
+                    lowest_job = f'{name}_well'
+                else:
+                    lowest_job = name
+                *_, last_row = self.db.select(name=f'{lowest_job}')
+                # The following refers to the conformer with lowest E + ZPE, 
+                # not the individual lowest.
+                lowest_energy = np.inf
+                lowest_zpe = np.inf
+                lowest_e_geom = last_row.positions
                 final_geoms = []  # list of all final conformer geometries
                 totenergies = []
                 frequencies = []
 
+                if all(status):  # if all conformers are invalid, 1 (different) or fail (-1)
+                    if self.qc.qc == 'gauss':
+                        ext = 'log'
+                    elif self.qc.qc == 'qchem':
+                        ext = 'out'
+                    elif self.qc.qc == 'nn_pes':
+                        ext = 'log'
+                    else:
+                        raise NotImplementedError(f'Code {self.qc.qc} not available.')
+                    copyfile(f'{lowest_job}.{ext}', f'conf/{name}_low.{ext}')
+                    mol = Atoms(symbols=last_row.symbols, positions=last_row.positions)
+                    data = {'energy': last_row.data.get('energy'),
+                            'frequencies': last_row.data.get('frequencies'),
+                            'zpe': last_row.data.get('zpe'),
+                            'status': last_row.data.get('status')}
+                    self.db.write(mol, name='conf/{}_low'.format(name), 
+                                  data=data)
+                    #logger.warning(f'All conformer optimizations failed for {name}.')
+
+                    return 1, lowest_conf, lowest_e_geom, last_row.data.get('energy'),\
+                           final_geoms, totenergies, frequencies, status
+
                 for ci in range(self.conf):
                     if status[ci] == 0:  # this is a valid confomer
                         add = ''
@@ -432,27 +472,25 @@ class Conformers:
                             frequencies.append(freq)
                         else:
                             frequencies.append(None)
-                        if lowest_totenergy == 0.:  # likely / hopefully the first sample was valid
-                            if ci != 0:
-                                logger.debug('For {} conformer 0 failed.'.format(name)) 
-                            err, freq = self.qc.get_qc_freq(job, self.species.natom)
+                        if lowest_energy is np.inf:
                             if self.species.natom > 1:
                                 # job fails if conformer freq array is empty
                                 if len(freq) > 0:
                                     if self.species.wellorts:
                                         if freq[0] >= 0.:
                                             err = -1
-                                        if self.species.natom > 2 and freq[1] <= 0.:
+                                        if self.species.natom > 2 and freq[1] <= -1 * self.imagfreq_threshold:
                                             err = -1
                                     else:
-                                        if freq[0] <= 0.:
+                                        if freq[0] <= -1 * self.imagfreq_threshold:
                                             err = -1
                                 else:
                                     logger.warning("Conformer {} failed due to empty freq array".format(ci))
                                     err = -1
                             if err == 0:
-                                lowest_totenergy = energy + zpe
-                        if energy + zpe < lowest_totenergy:
+                                lowest_energy = energy
+                                lowest_zpe = zpe
+                        if energy + zpe <= lowest_energy + lowest_zpe:
                             err, freq = self.qc.get_qc_freq(job, self.species.natom)
                             ratio = 0.8
                             # job fails if conformers freq array is empty
@@ -465,15 +503,16 @@ class Conformers:
                                     if self.species.natom > 2 and freq[1] <= 0.:
                                         err = -1
                                 else:
-                                    if freq[0] <= -50.:  # note that now we allow negative frequencies here as well
+                                    if freq[0] <= -1. * self.imagfreq_threshold:  # note that now we allow negative frequencies here as well
                                         err = -1
-                            else:
+                            elif self.species.natom > 1:
                                 logger.warning("Conformer {} failed due to empty freq array".format(ci))
                                 err = -1
                             if err == 0:
                                 lowest_job = job
                                 lowest_conf = str(ci).zfill(self.zf)
-                                lowest_totenergy = energy + zpe
+                                lowest_energy = energy
+                                lowest_zpe = zpe
                                 lowest_e_geom = geom
                         if err == -1:
                             status[ci] = 1  # make it invalid
@@ -488,31 +527,80 @@ class Conformers:
                             frequencies.append(np.zeros(self.species.natom * 3 - 6))
 
                 self.write_profile(status, final_geoms, totenergies)
-               
+
+                # Check if at least one conformer has the same enrgy as the L1 parent structure
+                if self.species.wellorts:
+                    *_, l1_last_row = self.db.select(name=self.species.name)
+                else:
+                    *_, l1_last_row = self.db.select(name=f'{self.species.name}_well')
+                l1energy = l1_last_row.data.get('energy') * constants.EVtoHARTREE
+                l1energy += l1_last_row.data.get('zpe')
+                if not any([abs(en - l1energy) < self.diffthrs * constants.KCALtoHARTREE for en in totenergies]): # 0.1 kcal/mol
+                    if self.print_warning:
+                        logger.warning(f'\tNone of {self.species.name} '
+                                       'conformers has the same energy as its '
+                                       'parent structure.')
+                        self.print_warning = False
+                    lowest_job = l1_last_row.name
+                    lowest_conf = 'low'
+                    lowest_e_geom = l1_last_row.positions
+                    lowest_energy = l1energy
+                
+                low_row = None
+                low_rows = self.db.select(name='conf/{}_low'.format(name))
+                for lrow in low_rows:
+                    low_row = lrow
                 try:
-                    copyfile('{}.log'.format(lowest_job), 'conf/{}_low.log'.format(name))
+                    if self.qc.qc == 'gauss':
+                        copyfile(f'{lowest_job}.log', f'conf/{name}_low.log')
+                    elif self.qc.qc == 'qchem':
+                        copyfile(f'{lowest_job}.out', f'conf/{name}_low.out')
+                    elif self.qc.qc == 'nn_pes':
+                        pass
+                    else:
+                        raise NotImplementedError(f'Code {self.qc.qc} not available.')
                     rows = self.db.select(name='{}'.format(lowest_job))
                     for row in rows:
                         row_last = row
-                    
                     mol = Atoms(symbols=row_last.symbols, positions=row_last.positions)
-                    self.db.write(mol, name='conf/{}_low'.format(name), 
-                             data={'energy': row_last.data.get('energy'),
-                             'frequencies': row_last.data.get('frequencies'),
-                             'zpe': row_last.data.get('zpe'),
-                             'status': row_last.data.get('status')})
+                    data = {'energy': row_last.data.get('energy'),
+                            'frequencies': row_last.data.get('frequencies'),
+                            'zpe': row_last.data.get('zpe'),
+                            'status': row_last.data.get('status')}
+                    if low_row and hasattr(low_row, 'data') \
+                            and all((np.all(data.get(k) == v) 
+                                     for k, v in low_row.data.items())):
+                        pass
+                    else:
+                        self.db.write(mol, name='conf/{}_low'.format(name), 
+                                      data=data)
                 except UnboundLocalError:
                     pass
-                
-                return 1, lowest_conf, lowest_e_geom, lowest_totenergy,\
+
+                return 1, lowest_conf, lowest_e_geom, lowest_energy,\
                        final_geoms, totenergies, frequencies, status
 
             else:
                 if wait:
                     time.sleep(1)
                 else:
-                    return 0, lowest_conf, np.zeros((self.species.natom, 3)), self.species.energy,\
-                           np.zeros((self.species.natom, 3)), np.zeros(1), np.zeros(1), np.zeros(1)
+                    return 0, lowest_conf, np.zeros((self.species.natom, 3)), \
+                           self.species.energy, np.zeros((self.species.natom, 3)), \
+                           np.zeros(1), np.zeros(1), np.zeros(1)
+
+
+    def add_new_conf_from_hir(self, geom) -> None:
+        """Generate new conformer found during hir calculations.
+
+        Args:
+            geom (_type_): cartesian geometry of the lowest energy point along all hir
+        """
+        self.qc.qc_conf(self.species, geom, self.conf, semi_emp=self.semi_emp)
+        if os.path.exists(f'conf/{self.get_name()}_low.log'):
+            logger.debug(f'Removing file conf/{self.get_name()}_low.log')
+            os.remove(f'conf/{self.get_name()}_low.log')
+        self.conf += 1
+
 
     def lowest_conf_info(self):
         """
@@ -522,11 +610,17 @@ class Conformers:
         name = self.get_name()
 
         job = 'conf/{}_low'.format(name)
-        err, energy = self.qc.get_qc_energy(job)
-        err, zpe = self.qc.get_qc_zpe(job)
-        err, geom = self.qc.get_qc_geom(job, self.species.natom)
+        try:
+            err, energy = self.qc.get_qc_energy(job)
+            err, zpe = self.qc.get_qc_zpe(job)
+            err, geom = self.qc.get_qc_geom(job, self.species.natom)
+        except ValueError:
+            _ = self.check_conformers()
+            err, energy = self.qc.get_qc_energy(job)
+            err, zpe = self.qc.get_qc_zpe(job)
+            err, geom = self.qc.get_qc_geom(job, self.species.natom)
                 
-        return geom, energy + zpe 
+        return geom, energy, zpe 
 
     def find_unique(self, conformers, energies, frequencies, valid, temp=None, boltz=None):
         """
@@ -555,17 +649,21 @@ class Conformers:
             # calculate the Gibbs free energy for all conformers
             # at T = temp, P = 101325 Pa
             gibbs = []
+            geo_type = 'nonlinear'
             for vi, val in enumerate(valid):
-                if frequencies[vi][0] > 0:
-                    vib_energies = [ff * invcm for ff in frequencies[vi]]  # convert to eV
+                if frequencies == [None]:
+                    vib_energies = [0]
+                    geo_type = 'monatomic'
                 else:
-                    vib_energies = [ff * invcm for ff in frequencies[vi][1:]]  # convert to eV
+                    vib_energies = [ff * invcm for ff in frequencies[vi] if ff > 0]  # convert to eV
+                    if np.shape(frequencies)[1] == 3 * len(self.species.atom) - 5:
+                        geo_type = 'linear'
                 potentialenergy = energies[vi] * Hartree  # convert to eV
                 atoms = Atoms(symbols=self.species.atom, positions=conformers[vi])
                 thermo = IdealGasThermo(vib_energies=vib_energies,
                                         potentialenergy=potentialenergy,
                                         atoms=atoms,
-                                        geometry='nonlinear',
+                                        geometry=geo_type,
                                         symmetrynumber=1, spin=(self.species.mult-1)/2)
                 gibbs.append(thermo.get_gibbs_energy(temperature=temp, pressure=101325., verbose=False))
 
