@@ -1,3 +1,3 @@
 message = "\nCopyright 2018 National Technology & Engineering Solutions of Sandia,\n\
 LLC (NTESS). Under the terms of Contract DE-NA0003525 with NTESS, \n\
-the U.S. Government retains certain rights in this software."
+the U.S. Government retains certain rights in this software.\n"
