@@ -4,6 +4,7 @@ import logging
 import time
 
 import rmsd
+import numpy as np
 
 from kinbot import frequencies
 from kinbot import geometry
@@ -29,33 +30,45 @@ class Optimize:
     4. Repeat steps 2-3 as long as lower energy structures are found
     """
 
-    def __init__(self, species, par, qc, wait=0):
+    def __init__(self, species, par, qc, wait=0, just_high=False):
+        """
+        just_high: ony do high_level calculation
+        """
         self.species = species
+
         try:
             delattr(self.species, 'cycle_chain')
         except AttributeError:
             logger.debug(f"{self.species.chemid} has no cycle_chain attribute "
                          f"to delete")
-        if self.species.wellorts:
+        if self.species.wellorts or par['cluster']:
             self.species.characterize(bond_mx=self.species.bond)
             self.name = str(self.species.name)
         else:
             self.species.characterize()
-            self.name = str(self.species.chemid)
+            self.name = self.species.name
         self.par = par
         self.qc = qc
         # wait for all calculations to finish before returning
         self.wait = wait
 
-        # status of the various parts
-        # -1: not yet started
-        #  0: running
-        #  1: finished
-        # -999:failed
-        self.scycconf = -1
-        self.sconf = -1
         self.shigh = -1
-        self.shir = -1
+        self.just_high = just_high
+
+        if not self.just_high:
+            # status of the various parts
+            # -1: not yet started
+            #  0: running
+            #  1: finished
+            # -999:failed
+            self.scycconf = -1
+            self.sconf = -1
+            self.shir = -1
+        else:
+            # Do not perform conformer search for vdW wells
+            self.scycconf = 1
+            self.sconf = 1
+            self.shir = 1
 
         # restart counter: number of times the high-level and hir calculations
         # has been restarted in case a lower energy structure has been found
@@ -120,7 +133,7 @@ class Optimize:
                     # do open chain part if cyclic (if there were any) and semi empirical (if requested) parts are done
                     if self.sconf == -1:
                         # open chain part has not started yet
-                        # if semi empirical conformer were searched for, start from those, 
+                        # if semi empirical conformer were searched for, start from those,
                         # else start from cyclic conformers
                         if self.par['semi_emp_conformer_search'] == 1:
                             self.species.confs.nconfs = 1
@@ -141,8 +154,11 @@ class Optimize:
                     if self.sconf == 0:
                         # conformational search is running
                         # check if the conformational search is done
-                        if self.skip_conf_check == 0 or self.par['multi_conf_tst'] or self.par['print_conf'] or self.par['calc_aie']:
-                            status, lowest_conf, geom, low_energy, conformers, energies, frequency_vals, valid =\
+                        if self.skip_conf_check == 0 \
+                                or self.par['multi_conf_tst'] \
+                                or self.par['print_conf'] \
+                                or self.par['calc_aie']:
+                            status, lowest_conf, geom, self.species.low_energy, conformers, energies, frequency_vals, valid =\
                                     self.species.confs.check_conformers(wait=self.wait)
                             if status == 1:
                                 self.species.conformer_geom, self.species.conformer_energy, \
@@ -156,16 +172,18 @@ class Optimize:
                                         self.par['multi_conf_tst_boltz'])
                                 logger.info(f'\tLowest energy conformer for species {self.name} is number {lowest_conf}')
                                 if self.par['multi_conf_tst'] or self.par['print_conf'] or self.par['calc_aie']:
-                                    logger.info(f'\tAt {self.par["multi_conf_tst_temp"]} K with {100*self.par["multi_conf_tst_boltz"]}% cutoff')
-                                    logger.info(f'\t\t the unique conformers for species {self.name} are {self.species.conformer_index}')
+                                    logger.info(f'\tAt {self.par["multi_conf_tst_temp"]} K '
+                                                f'with {100*self.par["multi_conf_tst_boltz"]}% '
+                                                f'cutoff the unique conformers for species {self.name} '
+                                                f'are {self.species.conformer_index}')
                                 # save lowest energy conformer as species geometry
                                 self.species.geom = geom
                                 # save lowest energy conformer energy
-                                self.species.energy = low_energy
+                                self.species.energy = self.species.low_energy
                                 # set conf status to finished
                                 self.sconf = 1
-                        elif self.skip_conf_check == 1:
-                            self.species.geom, self.species.energy = self.species.confs.lowest_conf_info()
+                        elif self.skip_conf_check == 1 and self.restart == 0:
+                            self.species.geom, self.species.energy, self.species.zpe = self.species.confs.lowest_conf_info()
                             logger.info('\tEnergy and geometry updated based on conf/{}_low file.'.format(self.name))
                             self.sconf = 1
 
@@ -174,8 +192,8 @@ class Optimize:
                 self.sconf = 1
             if self.sconf == 1:  # conf search is finished
                 # if the conformers were already done in a previous run
-                if self.par['conformer_search'] == 1:
-                    status, lowest_conf, self.species.geom, low_energy, conformers, energies, frequency_vals, valid = \
+                if self.par['conformer_search'] == 1 and not self.just_high and self.restart == 0:
+                    status, lowest_conf, self.species.geom, self.species.low_energy, conformers, energies, frequency_vals, valid = \
                         self.species.confs.check_conformers(wait=self.wait)
                         
                 while self.restart <= self.par['rotation_restart']:
@@ -193,8 +211,14 @@ class Optimize:
                                                           ext=f'_{str(conindx).zfill(4)}_high',
                                                           )
                             else:
-                                name = self.species.chemid
-                                self.qc.qc_opt(self.species, self.species.geom, high_level=1)
+                                if self.just_high:
+                                    name = self.species.name
+                                    self.qc.qc_opt(self.species, self.species.geom, high_level=1, do_vdW=True)
+                                    #Set conformer_zeroenergy to the species zero-energy if no conformer analysis has been done
+                                    self.species.conformer_zeroenergy = [self.species.energy + self.qc.get_qc_zpe(f"{self.species.name}_high", wait=0)[1]]
+                                else:
+                                    name = self.species.chemid
+                                    self.qc.qc_opt(self.species, self.species.geom, high_level=1)
                                 if self.par['multi_conf_tst']:
                                     for ci, conindx in enumerate(self.species.conformer_index):
                                         self.qc.qc_opt(self.species, 
@@ -220,7 +244,7 @@ class Optimize:
                                 logger.warning('High level optimization failed for {}'.format(self.name))
                                 self.shigh = -999
                             elif status == 'normal':
-                                self.compare_structures()  # this switches shigh to 0.5 or 1
+                                self.compare_structures()  # this switches shigh to 0.5 or 1 and updates the geometry
                         if self.shigh == 0.5:  # the top one was tested already and was ok
                             stati = [0] * len(self.species.conformer_index)
                             for ci, conindx in enumerate(self.species.conformer_index):
@@ -249,7 +273,7 @@ class Optimize:
                                 # hir is running
                                 # check if it is done:
                                 status = self.species.hir.check_hir(wait=self.wait)
-                                if status:
+                                if status:  # Finished correctly
                                     if len(self.species.hir.hir_energies) > 0:
                                         # check if along the hir potential a structure was found with a lower energy
                                         min_en = self.species.hir.hir_energies[0][0]
@@ -257,21 +281,24 @@ class Optimize:
                                         min_ai = -1
                                         for rotor in range(len(self.species.dihed)):
                                             for ai in range(self.species.hir.nrotation):
-                                                # use a 0.1kcal/mol cutoff for numerical noise
-                                                if self.species.hir.hir_energies[rotor][ai] < min_en - 1.6E-4:
-                                                    min_en = self.species.hir.hir_energies[rotor][ai]
-                                                    min_rotor = rotor
-                                                    min_ai = ai
+                                                # use a 0.1 kcal/mol cutoff for numerical noise
+                                                if self.species.hir.hir_status[rotor][ai] == 0:  # do not test for fails
+                                                    if self.species.hir.hir_energies[rotor][ai] < min_en - 1.6E-4:
+                                                        min_en = self.species.hir.hir_energies[rotor][ai]
+                                                        min_rotor = rotor
+                                                        min_ai = ai
                                         if min_rotor > -1:
                                             self.restart += 1
+                                            job = self.log_name(1, hir=1, r=min_rotor, s=min_ai)
+                                            diff = (self.species.hir.hir_energies[min_rotor][0] - 
+                                                    self.species.hir.hir_energies[min_rotor][min_ai]) * constants.AUtoKCAL 
                                             if self.restart < self.par['rotation_restart']:
                                                 # lower energy structure found
-                                                logger.warning(f'Lower energy conformer during HIR for {self.name} for rotor {min_rotor}. Restart #{self.restart}')
-                                                logger.debug('Rotor: ' + str(min_rotor))
-                                                logger.debug('Scan point: ' + str(min_ai))
-                                                job = self.log_name(1, hir=1, r=min_rotor, s=min_ai)
+                                                logger.warning(f'Lower energy conformer during HIR: {job}, diff = {np.round(diff, 2)} kcal/mol. Restart #{self.restart}')
 
                                                 err, self.species.geom = self.qc.get_qc_geom(job, self.species.natom)
+                                                # err, geom = self.qc.get_qc_geom(job, self.species.natom)
+                                                # self.species.confs.add_new_conf_from_hir(geom)
                                                 # delete the high_level log file and the hir log files
                                                 if os.path.exists(self.log_name(1) + '.log'):
                                                     logger.debug(f'Removing file {self.log_name(1)}.log')
@@ -285,7 +312,7 @@ class Optimize:
                                                 self.shigh = -1
                                                 self.shir = -1
                                             else:
-                                                logger.warning('Lower energy conformer, but reached max restart for {}'.format(self.name))
+                                                logger.warning(f'Lower energy during HIR: {job}, diff = {np.round(diff, 2)} kcal/mol. Reached max restart for this species.')
                                                 self.shir = 1
                                         else:
                                             self.shir = 1
@@ -309,7 +336,8 @@ class Optimize:
                 symmetry.calculate_symmetry(self.species)
 
                 # calculate the new frequencies with the internal rotations projected out
-                if self.par['multi_conf_tst'] == 0 and self.par['rotor_scan']:
+                if self.par['multi_conf_tst'] == 0 and self.par['rotor_scan'] \
+                        and not self.just_high:
                     fr_file = self.log_name(self.par['high_level'])
                     hess = self.qc.read_qc_hess(fr_file, self.species.natom)
                     if self.qc.qc == 'qchem':
@@ -333,20 +361,21 @@ class Optimize:
                             molp.create_molpro_input(bls=1)
                         else:
                             key = self.par['single_point_key']
-                            molp.create_molpro_input()
-                        molp.create_molpro_submit()
-                        status, molpro_energy = molp.get_molpro_energy(key)
+                            molp.create_molpro_input(do_vdW=self.just_high)
+                        if self.par['queuing'] != 'local':
+                            molp.create_molpro_submit(do_vdW=self.just_high)
+                        status, molpro_energy = molp.get_molpro_energy(key, do_vdW=self.just_high)
                         if status:
-                            self.species.energy = molpro_energy
+                            self.species.l3energy = molpro_energy
 
-                    if self.par['single_point_qc'] == 'orca':
+                    elif self.par['single_point_qc'] == 'orca':
                         orca = Orca(self.species, self.par)
                         key = self.par['single_point_key']
                         orca.create_orca_input()
                         orca.create_orca_submit()
                         status, orca_energy = orca.get_orca_energy(key)
                         if status:
-                            self.species.energy = orca_energy
+                            self.species.l3energy = orca_energy
 
                     self.delete_files()
 
@@ -388,11 +417,13 @@ class Optimize:
         else:
             names.append(self.name + '_well')
             names.append(self.name + '_well_high')
-            if self.par['high_level'] == 1:
+            if self.par['high_level'] == 1 and \
+               'prod' not in self.species.name:
                 for count in range(self.species.hir.nrotation):
                     for rot_num in range(self.par['nrotation']):
                         names.append('hir/' + self.name + '_hir_' + str(count) + '_' + str(rot_num).zfill(2))
-            if self.par['conformer_search'] == 1:
+            if self.par['conformer_search'] == 1 and \
+               'prod' not in self.species.name:
                 for count in range(self.species.confs.conf):
                     names.append('conf/' + self.name + '_' + str(count).zfill(zf))
                 for count in range(self.species.confs.cyc_conf):
@@ -424,7 +455,7 @@ class Optimize:
             return f'conf/{self.name}_{str(conf).zfill(4)}'
 
         name = str(self.name)
-        if not self.species.wellorts:
+        if not self.species.wellorts and not self.just_high:
             name += '_well'
         if high:
             name += '_high'
@@ -446,6 +477,7 @@ class Optimize:
                                 atom=self.species.atom,
                                 geom=new_geom)
         dummy.bond_mx()
+        dummy.calc_chemid()
 
         # comparing L1 and L2 geometries and imaginary mode if TS
         if self.species.wellorts:  # for TS we need reasonable geometry agreement and normal mode correlation
@@ -466,7 +498,7 @@ class Optimize:
             # maybe we need to do IRC at the high level as well...
             same_geom = ((geometry.matrix_corr(imagmode, imagmode_high) > 0.9) and \
                     (geometry.equal_geom(self.species, dummy, 0.3))) \
-                    or (geometry.equal_geom(self.species, dummy, 0.15))
+                    or (geometry.equal_geom(self.species, dummy, 0.2))
             if self.par['multi_conf_tst'] != 1:  # for now skipping this
                 p_coord = copy.deepcopy(self.species.geom)
                 q_coord = copy.deepcopy(dummy.geom)
@@ -487,24 +519,28 @@ class Optimize:
             else:
                 result_rmsd = 'not done'
             logger.info(f'\t{self.name} high level rmsd: {result_rmsd}, '\
-                         f'same(0.15): {geometry.equal_geom(self.species, dummy, 0.15)}, '\
-                         f'corr: {geometry.matrix_corr(imagmode, imagmode_high)}, '\
-                         f'same: {same_geom}')
+                        f'same(0.15): {geometry.equal_geom(self.species, dummy, 0.15)}, '\
+                        f'corr: {geometry.matrix_corr(imagmode, imagmode_high):.2f}, '\
+                        f'same: {same_geom}')
         else:
             same_geom = geometry.equal_geom(self.species, dummy, 0.1)
 
         # checking if L2 frequencies are okay
-        err, fr = self.qc.get_qc_freq(self.log_name(1, conf), self.species.natom)
+        err, freq = self.qc.get_qc_freq(self.log_name(1, conf), self.species.natom)
         if self.species.natom == 1:
             freq_ok = 1
-        elif len(fr) == 1 and fr[0] == 0:
+        elif len(freq) == 1 and freq[0] == 0:
             freq_ok = 0
-        elif self.species.wellorts == 0 and fr[0] > -50.:
+        elif self.species.wellorts == 0 and freq[0] > -1. * self.par['imagfreq_threshold']:
             freq_ok = 1
-            if fr[0] < 0.:
-                fr[0] *= -1.
-                logger.warning(f'Negative frequency {fr[0]} detected in {self.name}. Flipped.')
-        elif self.species.wellorts == 1 and fr[0] < 0. and fr[1] > 0.:
+            if freq[0] < 0.:
+                logger.warning(f'Negative frequency {freq[0]} cm-1 detected in '
+                            f'{self.name}. Flipped to {-freq[0]}.')
+                freq[0] *= -1.
+        elif self.species.wellorts == 1 \
+                and not (np.count_nonzero(np.array(freq) < 0) >= 3  # Three or more imag frequencies
+                        or np.count_nonzero(np.array(freq) < -1 * self.par['imagfreq_threshold']) >= 2  # More than one imaginary frequency beyond the threshold
+                        or np.count_nonzero(np.array(freq) < 0) == 0):  # No imaginary frequencies:
             freq_ok = 1
         else:
             freq_ok = 0
