@@ -1,9 +1,11 @@
 import logging
-import numpy as np
 import copy
 import math
 import itertools
 
+import numpy as np
+from ase.data import covalent_radii, atomic_numbers
+
 from kinbot import cheminfo
 from kinbot import constants
 from kinbot import find_motif
@@ -17,7 +19,8 @@ class StationaryPoint:
     This object contains the properties of wells.
     """
 
-    def __init__(self, name, charge, mult, smiles='', structure=None, natom=0, atom=None, geom=None, wellorts=0, fragA=None, fragB=None):
+    def __init__(self, name, charge, mult, smiles='', structure=None, natom=0, 
+                 atom=None, geom=None, wellorts=0, fragA=None, fragB=None):
         self.name = name
         self.mult = mult
         self.charge = charge
@@ -93,6 +96,49 @@ class StationaryPoint:
         if self.natom == 0:
             self.natom = len(atom)
 
+    @classmethod
+    def from_ase_atoms(cls, atoms, **kwargs):
+        """Builds a stationary point object from an ase.Atoms object.
+
+        Args:
+            atoms (ase.Atoms): The Atoms class from the ase library.
+
+        Returns:
+            StationaryPoint: A Stationary point object with the properties of 
+                the ase.Atoms properties
+        """
+        
+        name = kwargs.get('name')
+        charge = kwargs.get('charge')
+        mult = kwargs.get('mult')
+        geom = kwargs.get('geom')
+        symbols = kwargs.get('symbols')
+
+        if name is None:
+            name = 'StationaryPoint'
+        else:
+            name = kwargs["name"]
+
+        if charge is None:
+            if atoms.calc is None or 'charge' not in atoms.calc.parameters:
+                charge = sum(atoms.get_initial_charges())
+            else:
+                charge = atoms.calc.parameters['charge']
+
+        if mult is None:
+            if atoms.calc is None or 'mult' not in atoms.calc.parameters:
+                mult = cls.calc_multiplicity(cls, np.array(atoms.symbols))
+            else:
+                mult = atoms.calc.parameters['mult']
+
+        if 'geom' not in kwargs:
+            geom = atoms.positions
+
+        if 'symbols' not in kwargs:
+            symbols = list(atoms.symbols)
+
+        return cls(name, charge, mult, geom=geom, atom=symbols)
+
     def get_geom(self):
         """
         Method reads the structure and converts it to a geometry
@@ -161,6 +207,7 @@ class StationaryPoint:
         """ 
         Create bond matrix 
         Also create smiles if possible
+        Also create a bond matrix where it's only 1 or 0 (no double bonds etc.)
         """
         self.distance_mx()
         for i in range(self.natom):
@@ -168,19 +215,25 @@ class StationaryPoint:
                 if i == j:
                     continue
                 elif self.dist[i][j] < 0.5:
-                    err_msg = 'Incorrect geometry: Found an interatomic ' \
-                              'distance smaller than 0.5 Å.'
+                    err_msg = f'Incorrect geometry for {self.name}: Found an ' \
+                              'interatomic distance smaller than 0.5 Angstrom.'
                     logger.error(err_msg)
                     raise ValueError(err_msg)
 
         self.bond = np.zeros((self.natom, self.natom), dtype=int)
+        self.bonds = []  # unique list of bond matrices in case of resonance
 
         for i in range(self.natom):
             for j in range(self.natom):
                 if i == j: continue
                 atom_pair = [self.atom[i], self.atom[j]]
                 atom_pair = sorted(atom_pair)
-                if self.dist[i][j] < constants.st_bond[''.join(atom_pair)]:
+                try:
+                    bond_thresh = constants.st_bond[''.join(atom_pair)]
+                except KeyError:
+                    bond_thresh = 1.2 * (covalent_radii[atomic_numbers[atom_pair[0]]] 
+                                         + covalent_radii[atomic_numbers[atom_pair[1]]]) 
+                if self.dist[i][j] < bond_thresh:
                     self.bond[i][j] = 1
 
         max_bond = [constants.st_bond[self.atom[i]] for i in range(self.natom)]
@@ -231,7 +284,7 @@ class StationaryPoint:
                     if self.atom[i] == 'N':
                         if perm_rad[index][i] == 2:
                             perm_rad[index][i] = 0
-                    for ind2 in range(ind1, len(perm)):   
+                    for ind2 in range(ind1, len(perm)):
                         j = perm[ind2]
                         if perm_rad[index][i] > 0 and perm_rad[index][j] > 0 and perm_bond[index][i][j] > 0:
                             incr = 1
@@ -305,6 +358,12 @@ class StationaryPoint:
                     self.smiles = cheminfo.create_smi_from_geom(self.atom, self.geom)
                 except:
                     pass
+        self.bond01 = np.zeros((self.natom, self.natom), dtype=int)
+        for ri, row in enumerate(self.bond):
+            for bii, bi in enumerate(row):
+                if bi > 0:
+                    self.bond01[ri][bii] = 1
+
         return 0
 
     def make_extra_bond(self, parts, maps):
@@ -318,6 +377,8 @@ class StationaryPoint:
         n is the nth fragment and i is the ith atom in this fragment 
         """
 
+        if len(parts) == 1:
+            return -1 -1
         mindist = 100.  # large initial value
         for i, cooi in enumerate(parts[0].geom): 
             for j, cooj in enumerate(parts[1].geom):
@@ -328,13 +389,49 @@ class StationaryPoint:
                     pivot2 = maps[1][j]
         self.bond[pivot1][pivot2] = 1
         self.bond[pivot2][pivot1] = 1
+        for b in self.bonds:
+            b[pivot1][pivot2] = 1
+            b[pivot2][pivot1] = 1
+        return pivot1, pivot2
+
+    def make_hbonds(self):
+        """
+        Add all hydrogen bonds. Criteria:
+        - has to be an X1-H-X2 pattern
+        - X1-H is already bonded 
+        - X2-H is less than 1.8 A
+        - X2 has free valence
+        - H only has one bond (to X1)
+        - X1-H-X2 angle is > 150 degrees
+        - X1 and X2 have to be one of O, N, F
+        """
+
+        self.hbonds = []
+        for hi, symb in enumerate(self.atom):
+            if symb == 'H':
+                if np.sum(self.bond[hi]) == 1:  # X1-H is already bonded 
+                    if self.atom[np.where(self.bond[hi] == 1)] in ['O', 'N', 'F']:  # H only has one bond (to X1), and X1 has to be one of O, N, F
+                        x1 = np.where(self.bond[hi] == 1)
+                        for x2, symb in enumerate(self.atom):
+                            if symb in ['O', 'N', 'F']:  # X2 has to be one of O, N, F
+                                if self.bond[hi][x2] == 0:  # they are not yet bonded
+                                    if np.sum(self.bond[x2]) <= constants.st_bond[symb]:  # X2 has free valence
+                                        if self.dist[hi][x2] < 1.8:  # X2-H is less than 1.8 A
+                                            if geometry.calc_angle(self.geom[x1],self.geom[hi],self.geom[x2]) > np.pi * 150. / 180.: 
+                                                self.bond[hi][x2] = 1
+                                                self.bond[x2][hi] = 1
+                                                logger.info(f'Adding H-bonds between {hi + 1} and {x2 + 1}')
+                                                self.hbonds.append([hi, x2])
+                                                for b in self.bonds:
+                                                    b[hi][x2] = 1
+                                                    b[x2][hi] = 1
         return 0
 
     def calc_multiplicity(self, atomlist):
         """ 
         1 = singlet, 2 = doublet, 3 = triplet, etc.
         """
-        if all([element == 'O' for element in atomlist]):
+        if atomlist.tolist() == ['O'] or atomlist.tolist() == ['O', 'O']:
             return 3 # O and O2 are triplet
         if len(atomlist) == 1 and atomlist[0] == 'S':
             return 3 # S is triplet
@@ -351,73 +448,93 @@ class StationaryPoint:
 
         return 1 + mult % 2
 
-    def start_multi_molecular(self):
+    def start_multi_molecular(self, vary_charge=False, bond_mx=None):
         """
         Iterative method to find all the separate products from a bond matrix
+        vary_charge: to identify fragments with charge on either of them
+        bond_mx: assume a known bond_mx at the characterization - needed in cluster mode for H bonds
         """
-        bond = copy.deepcopy(self.bond)
-
-        status = [0 for i in range(self.natom)]  # 1: part of a molecule, 0: not part of a molecule
-        atoms = [i for i in range(self.natom)]
-        mols = []  # list of stationary_pt objects for the parts 
-        atomlist = np.asarray(self.atom)
+        bond_mtx = copy.deepcopy(self.bond)
+        assigned_atoms = [0 for i in range(self.natom)]  # 1: part of the fragment, 0: not part of a fragment
+        atom_indices = [i for i in range(self.natom)]
+        st_pt_prodlist = []  # list of stationary_pt objects for the fragments 
+        symbols = np.asarray(self.atom)
         maps = []  # this maps the original atom numbering onto the fragments' numbers
 
         while 1:
-            if any([status[i] == 0 for i in range(len(status))]):
-                # reduce the bond matrix to the atoms that have a 0 as status
-                bondi = [[bond[i][j] for j in range(len(status)) if status[j] == 0] for i in range(len(status)) if status[i] == 0]
-                at = [atoms[i] for i in range(len(status)) if status[i] == 0]
-                natomi = len(at)
-                fragi = [0 for i in range(self.natom)]
-                if natomi == 1:
-                    #this is a molecule containing only one atom
-                    fragi[at[0]] = 1
-                    atomi = [at[0]]
-                    bool = 0
+            if not all(assigned_atoms):
+                # Work with the unassigned atoms (have 0 in assigned_atoms)
+                unass_bond_mtx = [[bond_mtx[i][j] for j in range(len(assigned_atoms)) if assigned_atoms[j] == 0] 
+                                  for i in range(len(assigned_atoms)) if assigned_atoms[i] == 0]
+                unass_idx = [idx for idx in atom_indices if assigned_atoms[idx] == 0]
+                frag_num_atoms = len(unass_idx)
+                frag_assg = [0] * self.natom
+                if frag_num_atoms == 1:
+                    #this is a fragment containing only one atom
+                    frag_assg[unass_idx[0]] = 1
+                    is_multifrag = 0
                 else:
-                    natomi = len(at)
-                    bool, sta = self.extract_next_mol(natomi,bondi)
-                    atomi = [at[i] for i in range(natomi) if sta[i] == 1]
-                    for i in range(len(sta)):
-                        if sta[i] == 1:
-                            status[at[i]] = 1
-                            fragi[at[i]] = 1
+                    is_multifrag, new_assign = self.extract_next_mol(frag_num_atoms, unass_bond_mtx)
+                    for i, assigned in enumerate(new_assign):
+                        if assigned == 1:
+                            assigned_atoms[unass_idx[i]] = 1
+                            frag_assg[unass_idx[i]] = 1
 
-                if not bool and len(mols) == 0:
+                if not is_multifrag and len(st_pt_prodlist) == 0:
                     #the bond matrix corresponds to one molecule only
                     try:
                         delattr(self, 'cycle_chain')
                     except AttributeError:
                         pass
-                    self.characterize()  
+                    self.characterize(bond_mx=self.bond)
                     self.name = str(self.chemid)
-                    mols.append(self)
+                    st_pt_prodlist.append(self)
                     break
-                geomi = np.asarray(self.geom)[np.where(np.asarray(fragi) == 1)]
-                natomi = np.sum(fragi)
-                atomi = atomlist[np.where(np.asarray(fragi) == 1)]
-                multi = self.calc_multiplicity(atomi)
-                chargei = self.charge # todo
-                moli = StationaryPoint('prod_%i'%(len(mols)+1), chargei, multi, atom=atomi, natom=natomi, geom=geomi)
-                moli.characterize()  
-                moli.calc_chemid()
-                moli.name = str(moli.chemid)
+                frag_geom = np.asarray(self.geom)[np.where(np.asarray(frag_assg) == 1)]
+                frag_num_atoms = np.sum(frag_assg)
+                frag_symbols = symbols[np.where(np.asarray(frag_assg) == 1)]
 
-                mols.append(moli)
+                if vary_charge and self.charge != 0:
+                    multiply = 2  # do two versions
+                else:
+                    multiply = 1
+                
+                for ch in range(multiply):
+                    # ch == 0 is neutral, ch == 1  is ion case
+                    multi = self.calc_multiplicity(frag_symbols)
+                    # on the second pass, adjust the multiplicity
+                    if ch == 1 and multi == 1:
+                        multi = 2
+                    elif ch == 1 and multi == 2:
+                        multi = 1
+                    if ch == 1 and multi == 3:
+                        multi = 2
 
-                numbering = np.asarray(range(self.natom))
-                # the original atom numbers in the correct order in the fragments, it's a map
-                mapi = numbering[np.where(np.asarray(fragi) == 1)]  
-                maps.append(mapi)
+                    if ch == 0:   
+                        chargei = 0
+                    elif ch == 1:
+                        chargei = self.charge 
+                    moli = StationaryPoint('prod_%i'%(len(st_pt_prodlist)+1), 
+                                           chargei, multi, atom=frag_symbols, 
+                                           natom=frag_num_atoms, geom=frag_geom)
+                    moli.characterize()  
+                    moli.calc_chemid()
+                    moli.name = str(moli.chemid)
 
-                if bool:
+                    st_pt_prodlist.append(moli)
+
+                    numbering = np.asarray(range(self.natom))
+                    # the original atom numbers in the correct order in the fragments, it's a map
+                    mapi = numbering[np.where(np.asarray(frag_assg) == 1)]  
+                    maps.append(mapi)
+
+                if is_multifrag:
                     continue 
                 else:
                     #reached the end, return the molecules
                     break
 
-        return mols, maps
+        return st_pt_prodlist, maps
 
     def extract_next_mol(self, natom, bond):
         """
@@ -429,7 +546,7 @@ class StationaryPoint:
 
         max_step = 1000
         status = [0 for i in range(natom)] # 1: parent, -1: child, 0: not yet checked
-        edge = [0 for i in range(natom*(natom-1)//2)] # the ordered list of edges in the graph, 0-1, 0-2, ..., 1-2, 1-3, ...
+        edge = [0 for i in range(natom * (natom - 1) // 2)] # the ordered list of edges in the graph, 0-1, 0-2, ..., 1-2, 1-3, ...
         chain = [] # steps made during the search
 
         n_connect = np.count_nonzero(bond) // 2 # number of atom connections
@@ -530,7 +647,6 @@ class StationaryPoint:
                       
         for i in range(self.natom):
             self.start_id(i) 
-        
         for i in range(self.natom):
             self.chemid += self.atomid[i]
         self.chemid *= 10
@@ -544,7 +660,6 @@ class StationaryPoint:
         i is the index for the atom to start at.
         """
         
-        
         visit = [0 for k in range(self.natom)]
         depth = 0
         atomid = int(0)
@@ -584,7 +699,7 @@ class StationaryPoint:
         """ 
         Identify unique rotatable bonds in the structure 
         No rotation around ring bonds and double and triple bonds.
-        If all is set to 1, then redundant dihedrals are also found.
+        If findall is set to 1, then redundant dihedrals are also found.
         """
         
         self.calc_chemid()
@@ -593,7 +708,7 @@ class StationaryPoint:
         if len(self.bonds) == 0:
             self.bonds = [self.bond]
         self.dihed = []
-        self.dihed_all = []
+        self.dihed_allrot = []
         hit = 0
 
         if self.natom < 4: return 0
@@ -616,10 +731,40 @@ class StationaryPoint:
                                             self.dihed.append([a, b, c, d])
                                             hit = 1 
                                         else:
-                                            self.dihed_all.append([a, b, c, d])
+                                            self.dihed_allrot.append([a, b, c, d])
 
         return 0
 
+    def find_alldihedral(self, collinear_cutoff=None): 
+        """ 
+        Identify all dihedrals
+        Also keeps the ones that are with linear angles
+        This is to be used in frozen calculations
+        """
+        
+        self.calc_chemid()
+        if not hasattr(self, 'cycle_chain'):
+            self.find_cycle()
+        if len(self.bonds) == 0:
+            self.bonds = [self.bond]
+        self.dihed = []
+        self.dihed_all = []
+
+        if self.natom < 4: return 0
+
+        # a-b-c-d, rotation around b-c
+        for b in range(self.natom):
+            for c in range(b, self.natom):
+                if self.bond[b][c] > 0:
+                    for a in range(self.natom):
+                        if self.bond[a][b] > 0 and a != c:
+                            for d in range(self.natom):
+                                if self.bond[c][d] > 0 and d != b:
+                                    dihedral_angle, warning = geometry.calc_dihedral(self.geom[a], self.geom[b], self.geom[c], self.geom[d], collinear_cutoff=collinear_cutoff)
+                                    if not warning:
+                                        self.dihed_all.append([a, b, c, d])
+        return 0
+
     def find_conf_dihedral(self):
         """
         Just keep those rotatable bonds that are needed for conformer search.
@@ -772,7 +917,6 @@ class StationaryPoint:
 
         return 0
     
-
     def calc_chiral(self):
         """
         Calculate self.chiral. 0 if non-chiral, +1 or -1 if chiral. Each atom gets a label like this.
@@ -815,7 +959,6 @@ class StationaryPoint:
                                 self.chiral[center] = self.calc_chiral_hand(self.geom[center], positions, atids)
         return self.chiral
 
-
     def calc_chiral_hand(self, center, ligands, atomids):
         """
         Calculate the handedness of a chiral center. 
@@ -860,7 +1003,6 @@ class StationaryPoint:
         
         return hand
 
-
     def find_linear(self):
         self.linear = []
         for ati in range(self.natom):
@@ -879,7 +1021,6 @@ class StationaryPoint:
 
         return
 
-
     def calc_maxbond(self):
         """
         Needed to find rigid segments
@@ -892,6 +1033,12 @@ class StationaryPoint:
 
         return
 
+    def reset_order(self) -> None:
+        """Reinitialize the bond matrix and the radicals 
+        """
+        self.bonds = []
+        self.rads = []
+        self.characterize()
 
 def main():
     """
