@@ -1,8 +1,7 @@
 from ase import Atoms
-# from ase.calculators.gaussian import Gaussian
 from ase.db import connect
 
-from kinbot.ase_modules.calculators.gaussian import Gaussian  # New
+from kinbot.ase_modules.calculators.gaussian import Gaussian
 from kinbot import reader_gauss
 from kinbot.utils import iowait
 
@@ -26,7 +25,7 @@ except RuntimeError:
     try:
         iowait(logfile, 'gauss')
         mol.positions = reader_gauss.read_geom(logfile, mol)
-        kwargs = reader_gauss.correct_kwargs(logfile, kwargs)
+        #kwargs = reader_gauss.correct_kwargs(logfile, kwargs)
         mol.calc = Gaussian(**kwargs)
         e = mol.get_potential_energy()  # use the Gaussian optimizer
         iowait(logfile, 'gauss')
